# opzet/kernel.py
#
#

""" kernel module. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from opzet.utils.opts import make_opts, opts_defs
from opzet.utils.other import hello, show_eggs
from opzet.defines import GREEN, ENDC
from opzet.cfg import cfg_kernel
from opzet.thing import Thing
from opzet.plugs import Plugs
from opzet.log import init

import logging
import time
import sys
import os

## PATH

def opzetpath():
    for path in sys.path:
        if "opzet" in path and "egg" in path: yield path  

## OPZETEL

class Kernel(Plugs):

    def boot(zelf, *args, **kwargs):
        """
             Enable runtime services of the OPZET bot.

             >>> kernel = Kernel()
             >>> kernel.boot("OPZET")
             >>> kernel.wait()

        """ 
        zelf._start = time.time()
        opts, arglist = make_opts(opts_defs)
        cfg.update(cfg_kernel)
        cfg.update(vars(opts))
        for key,value in kwargs.items():
            if not getattr(cfg , key, ""): cfg[key] = value
        if not cfg.workdir: cfg.workdir = os.path.expanduser("~/opzet.data")
        cfg.runargs = arglist
        init(cfg.loglevel, colors=cfg.colors)
        if cfg.shell: hello(*args, **cfg)
        if cfg.path:
            for path in sys.path: print(path)
            print()
        plugs.boot(cfg.packages)
        if "all" in cfg.modules: plugs.loaddir(os.getcwd())
        else:
            for mod in cfg.modules.split(","):
                if not mod: continue
                try: plugs.load(mod)
                except Exception as ex: print(ex)
        plugs.init("system")
        zelf.ready()

    def shutdown(zelf, *args, **kwargs):
        logging.warn("# shutdown")
        plugs.shutdown(cfg.packages)
        kernel.exit()
        kernel.stop()
        time.sleep(0.01)

## DEFINITIES

kernel = Kernel()
plugs = Plugs()
cfg = Thing()
fleet = []
