# opzet/plugs/test.py
# encoding: utf-8
#
#

""" test plugin. """

## IMPORTS

from opzet.kernel import kernel, cfg
from opzet.utils.name import typed
from opzet.bots import TestBot
from opzet.thing import Thing

import logging
import time

## DEFINES

exclude = ["fetch", "test"]

## COMMANDS

def forcedreconnect(event):
    """ no arguments - do a forced reconnect. """
    bot = event._target
    try: bot._sock.shutdown(2)
    except: event.reply("%s bot doesn't have a _sock attribute" % typed(bot))

kernel.register('test.forcedreconnect', forcedreconnect)

def forcedexception(event):
    """ no arguments - raise a exception. """
    event.reply("raising exception.")
    raise Exception('test exception')

kernel.register('test.forcedexception', forcedexception)

def wrongxml(event):
    """ no arguments - try sending borked xml. """
    event.reply('sending bork xml')
    bot = event._target
    bot._raw('<message asdfadf/>')

kernel.register('test.wrongxml', wrongxml)

def xmlescape(event):
    """ no arguments - try sending borked xml. """
    event.reply('sending xml')
    bot = event._target
    event.reply('<message asdfadf/>')

kernel.register('test.xmlescape', xmlescape)

def unicode(event):
    """ no arguments - send unicode test down the output paths. """
    outtxt = u"Đíť ìš éèñ ëņċøďıńğŧęŝţ· .. にほんごがはなせません .. ₀0⁰₁1¹₂2²₃3³₄4⁴₅5⁵₆6⁶₇7⁷₈8⁸₉9⁹ .. ▁▂▃▄▅▆▇▉▇▆▅▄▃▂▁ .. .. uǝʌoqǝʇsɹǝpuo pɐdı ǝɾ ʇpnoɥ ǝɾ"
    event.reply(outtxt)

kernel.register('test.unicode', unicode)

def deadline(event):
    """ no arguments - slee for 40 seconds in the mainloop. """
    try: nrseconds = int(event.parsed.rest)
    except: event.reply("need number of seconds to sleep.") ; return
    event.reply('starting %s sec sleep' % nrseconds)
    time.sleep(nrseconds)

kernel.register('test.deadline', deadline)

def html(event):
    """ arguments: [<txt>] - test the html=True option to event.reply(). """
    event.reply('<span style="font-family: fixed; font-size: 10pt"><b>YOOOO BROEDERS</b></span>')

kernel.register('test.html', html)

def testcmnds(event):
    if not kernel.allowed(event.origin): event.reply("ENOTALLOWED %s" % event.origin) ; return
    bot = TestBot()
    bot.workdir = "test.data"
    try: nrloops = int(event.parsed.rest)
    except: nrloops = 1
    for x in range(nrloops):
        for cmnd in sorted(list(kernel.names())):
            go = True
            for c in exclude:
                if c in cmnd: go = False
            if not go: continue
            e = bot.event("!" + cmnd + " cfg")
            logging.warn("< %s" % e.txt)
            event.reply("cmnds is %s" % e.txt)
            bot.dispatch(e, nocollect=True)

kernel.register("test.testcmnds", testcmnds)