# opzet/services/rss.py
#
#

""" rss module. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from opzet.utils.url import get_url, strip_html, need_redirect
from opzet.utils.lock import locked
from opzet.utils.time import short_date
from opzet.errors import error
from opzet.clock import Repeater
from opzet.kernel import kernel
from opzet.thing import Thing
from opzet.cfg import cfg_rss

import logging
import time

## DEFINES

seen = Thing()
seen.list = []
seen.results = "rss"
seen.prefix = "results"

## DISPLAY

def display(*args, **kwargs):
    entry = args[0]
    result = ""
    for key in cfg_rss.keys_list:
        if entry.get(key, None): result += "%s - " % entry[key]
    return result[:-3]

## GET

@locked
def get(url):
    import opzet.extra.feedparser as fp
    result = []
    logging.info("^ rss %s" % url)
    try:
        resp = get_url("GET", url)
        url = need_redirect(resp)
        if url: resp = get_url("GET", url)
        data = resp.read()
        if data: result = fp.parse(data)
    except Exception as ex: error(url) ; return
    for entry in result["entries"]: yield Thing(entry)

## FETCH

def fetch(*args, **kwargs):
    global seen
    obj = args[0]
    counter = 0
    for o in get(obj.rss):
        if o.link in seen.list: logging.info("! skip %s" % o.link) ; continue
        seen.list.append(o.link)
        s = o.slice(cfg_rss.keys_list)
        s.services = "rss"
        s.prefix = "rss"
        s.short = short_date(time.ctime())
        kernel.announce(display(s))
        counter += 1
    logging.warn("# %s fetched from %s" % (counter, obj.rss))

def rss_fetch(*args, **kwargs):
    for fn in kernel.all("rss"):
        obj = Thing().load(fn)
        if "deleted" in obj and obj.deleted: continue
        if "rss" not in obj: continue
        if "http" not in obj.rss: continue
        kernel.put(fetch, obj)
        time.sleep(0.1)

kernel.register("rss.fetch", rss_fetch)

## WRITE

def write_rss(event):
    o = Thing(event)
    o.prefix = "rss"  
    o.rss = event.parsed.rest
    o.services = "rss"
    path = o.save()  
    event.ok()

kernel.register("write.rss", write_rss)

## SYSTEM

def init(*args, **kwargs):
    global seen
    last_rss = kernel.last("results", "rss")
    if last_rss: seen = last_rss
    cfg = kernel.last("cfg", "rss")
    if not cfg: cfg = cfg_rss ; cfg.save()
    rss = Repeater(rss_fetch, 300)
    kernel.put(rss.start)

def sync(*args, **kwargs): seen.sync()
